@@ -7,6 +7,8 @@ from __future__ import print_function
 from __future__ import unicode_literals
 
 import contextlib
+import typing
+
 import six
 import pkg_resources
 
@@ -14,12 +16,17 @@ from .base import Opener
 from .errors import UnsupportedProtocol, EntryPointError
 from .parse import parse_fs_url
 
+if False:  # typing.TYPE_CHECKING
+    from typing import Iterator, List, Text, Tuple, Union
+    from ..base import FS
+
 
 class Registry(object):
     """A registry for `Opener` instances.
     """
 
     def __init__(self, default_opener='osfs'):
+        # type: (Text) -> None
         """Create a registry object.
 
         Arguments:
@@ -29,14 +36,15 @@ class Registry(object):
 
         """
         self.default_opener = default_opener
-        self._protocols = None
-
+        self._protocols = None  # type: Union[None, List[Text]]
 
     def __repr__(self):
+        # type: () -> Text
         return "<fs-registry {!r}>".format(self.protocols)
 
     @property
     def protocols(self):
+        # type: () -> List[Text]
         """`list`: the list of supported protocols.
         """
         if self._protocols is None:
@@ -48,6 +56,7 @@ class Registry(object):
         return self._protocols
 
     def get_opener(self, protocol):
+        # type: (Text) -> Opener
         """Get the opener class associated to a given protocol.
 
         Arguments:
@@ -103,11 +112,13 @@ class Registry(object):
         return opener_instance
 
     def open(self,
-             fs_url,
-             writeable=True,
-             create=False,
-             cwd=".",
-             default_protocol='osfs'):
+             fs_url,                    # type: Text
+             writeable=True,            # type: bool
+             create=False,              # type: bool
+             cwd=".",                   # type: Text
+             default_protocol='osfs'    # type: Text
+             ):
+        # type: (...) -> Tuple[FS, Text]
         """Open a filesystem from a FS URL.
 
         Returns a tuple of a filesystem object and a path. If there is
@@ -119,7 +130,7 @@ class Registry(object):
                 writeable.
             create (bool, optional): `True` if the filesystem should be
                 created if it does not exist.
-            cwd (str, optional): The current working directory, or `None`.
+            cwd (str): The current working directory.
 
         Returns:
             (FS, str): a tuple of ``(<filesystem>, <path from url>)``
@@ -145,11 +156,13 @@ class Registry(object):
         return open_fs, open_path
 
     def open_fs(self,
-                fs_url,
-                writeable=False,
-                create=False,
-                cwd=".",
-                default_protocol='osfs'):
+                fs_url,                     # type: Text
+                writeable=False,            # type: bool
+                create=False,               # type: bool
+                cwd=".",                    # type: Text
+                default_protocol='osfs'     # type: Text
+                ):
+        # type: (...) -> FS
         """Open a filesystem from a FS URL (ignoring the path component).
 
         Arguments:
@@ -158,8 +171,8 @@ class Registry(object):
                 be writeable.
             create (bool, optional): `True` if the filesystem should be
                 created if it does not exist.
-            cwd (str, optional): The current working directory (generally
-                only relevant for OS filesystems).
+            cwd (str): The current working directory (generally only
+                relevant for OS filesystems).
             default_protocol (str): The protocol to use if one is not
                 supplied in the FS URL (defaults to ``"osfs"``).
 
@@ -181,7 +194,13 @@ class Registry(object):
         return _fs
 
     @contextlib.contextmanager
-    def manage_fs(self, fs_url, create=False, writeable=False, cwd='.'):  # noqa: D300,D301
+    def manage_fs(self,
+                  fs_url,           # type: Union[FS, Text]
+                  create=False,     # type: bool
+                  writeable=False,  # type: bool
+                  cwd='.'           # type: Text
+                  ):
+        # type: (...) -> Iterator[FS]
         """Get a context manager to open and close a filesystem.
 
         Arguments:
@@ -199,7 +218,7 @@ class Registry(object):
 
         Example:
             >>> def print_ls(list_fs):
-            ...     \"\"\"List a directory.\"\"\"
+            ...     '''List a directory.'''
             ...     with manage_fs(list_fs) as fs:
             ...         print(' '.join(fs.listdir()))
 
