@@ -16,6 +16,7 @@ import os
 import platform
 import stat
 import sys
+import typing
 
 import six
 
@@ -23,8 +24,8 @@ try:
     from os import scandir
 except ImportError:
     try:
-        from scandir import scandir
-    except ImportError:  # pragma: nocover
+        from scandir import scandir    # type: ignore
+    except ImportError:  # pragma: no cover
         scandir = None
 
 from . import errors
@@ -39,6 +40,18 @@ from .error_tools import convert_os_errors
 from .mode import Mode, validate_open_mode
 from .errors import NoURL
 
+if False:  # typing.TYPE_CHECKING
+    from typing import (
+        Any, BinaryIO, Callable, Collection, Dict,
+        Iterator, IO, List, Optional, SupportsInt,
+        Text, Tuple)
+    from .base import _OpendirFactory
+    from .info import RawInfo
+    from .subfs import SubFS
+
+    _O = typing.TypeVar('_O', bound='OSFS')
+
+
 log = logging.getLogger('fs.osfs')
 
 
@@ -52,11 +65,11 @@ class OSFS(FS):
     Arguments:
         root_path (str or ~os.PathLike): An OS path or path-like object to
             the location on your HD you wish to manage.
-        create (bool, optional): Set to `True` to create the root directory
-            if it does not already exist, otherwise the directory should exist
-            prior to creating the ``OSFS`` instance (default `False`).
-        create_mode (int, optional): The permissions that will be used to
-            create the directory if ``create`` is `True` and the path doesn't
+        create (bool): Set to `True` to create the root directory if it
+            does not already exist, otherwise the directory should exist
+            prior to creating the ``OSFS`` instance (defaults to `False`).
+        create_mode (int): The permissions that will be used to create
+            the directory if ``create`` is `True` and the path doesn't
             exist, defaults to ``0o777``.
 
     Raises:
@@ -71,21 +84,26 @@ class OSFS(FS):
     """
 
     def __init__(self,
-                 root_path,
-                 create=False,
-                 create_mode=0o777):
+                 root_path,             # type: Text
+                 create=False,          # type: bool
+                 create_mode=0o777      # type: SupportsInt
+                 ):
+        # type: (...) -> None
         """Create an OSFS instance.
         """
         super(OSFS, self).__init__()
-        root_path = fsdecode(fspath(root_path))
+        if isinstance(root_path, bytes):
+            root_path = fsdecode(root_path)
+        self.root_path = root_path
+        _root_path = fsdecode(fspath(root_path))
         _root_path = os.path.expanduser(os.path.expandvars(root_path))
         _root_path = os.path.normpath(os.path.abspath(_root_path))
-        self.root_path = _root_path
+        self._root_path = _root_path
 
         if create:
             try:
                 if not os.path.isdir(_root_path):
-                    os.makedirs(_root_path, mode=create_mode)
+                    os.makedirs(_root_path, mode=int(create_mode))
             except OSError as error:
                 raise errors.CreateFailed(
                     'unable to create {} ({})'.format(root_path, error),
@@ -105,7 +123,7 @@ class OSFS(FS):
             'virtual': False,
         }
 
-        if _WINDOWS_PLATFORM:  # pragma: nocover
+        if _WINDOWS_PLATFORM:  # pragma: no cover
             _meta["invalid_path_chars"] =\
                 ''.join(six.unichr(n) for n in range(31)) + '\\:*?"<>|'
         else:
@@ -120,26 +138,32 @@ class OSFS(FS):
                 )
 
     def __repr__(self):
+        # type: () -> str
         _fmt = "{}({!r})"
-        return _fmt.format(self.__class__.__name__,
-                           self.root_path)
+        _class_name = self.__class__.__name__
+        return _fmt.format(_class_name, self.root_path)
 
     def __str__(self):
+        # type: () -> str
         fmt = "<{} '{}'>"
-        return fmt.format(self.__class__.__name__.lower(),
-                          self.root_path)
+        _class_name = self.__class__.__name__
+        return fmt.format(_class_name.lower(), self.root_path)
 
     def _to_sys_path(self, path):
+        # type: (Text) -> Text
         """Convert a FS path to a path on the OS.
         """
-        sys_path = os.path.join(
-            self.root_path,
-            path.lstrip('/').replace('/', os.sep)
+        sys_path = fsencode(
+            os.path.join(
+                self._root_path,
+                path.lstrip('/').replace('/', os.sep)
+            )
         )
         return sys_path
 
     @classmethod
     def _make_details_from_stat(cls, stat_result):
+        # type: (os.stat_result) -> Dict[Text, object]
         """Make a *details* info dict from an `os.stat_result` object.
         """
         details = {
@@ -163,25 +187,26 @@ class OSFS(FS):
 
     @classmethod
     def _make_access_from_stat(cls, stat_result):
+        # type: (os.stat_result) -> Dict[Text, object]
         """Make an *access* info dict from an `os.stat_result` object.
         """
-        access = {}
+        access = {}   # type: Dict[Text, object]
         access['permissions'] = Permissions(
             mode=stat_result.st_mode
         ).dump()
-        access['gid'] = stat_result.st_gid
-        access['uid'] = stat_result.st_uid
+        access['gid'] = gid = stat_result.st_gid
+        access['uid'] = uid = stat_result.st_uid
         if not _WINDOWS_PLATFORM:
             import grp
             import pwd
             try:
-                access['group'] = grp.getgrgid(access['gid']).gr_name
-            except KeyError:  # pragma: nocover
+                access['group'] = grp.getgrgid(gid).gr_name
+            except KeyError:  # pragma: no cover
                 pass
 
             try:
-                access['user'] = pwd.getpwuid(access['uid']).pw_name
-            except KeyError:  # pragma: nocover
+                access['user'] = pwd.getpwuid(uid).pw_name
+            except KeyError:  # pragma: no cover
                 pass
         return access
 
@@ -197,6 +222,7 @@ class OSFS(FS):
 
     @classmethod
     def _get_type_from_stat(cls, _stat):
+        # type: (os.stat_result) -> ResourceType
         """Get the resource type from an `os.stat_result` object.
         """
         st_mode = _stat.st_mode
@@ -208,30 +234,30 @@ class OSFS(FS):
     # --------------------------------------------------------
 
     def _gettarget(self, sys_path):
+        # type: (Text) -> Optional[Text]
         try:
-            target = os.readlink(sys_path)
+            target = os.readlink(fsencode(sys_path))
         except OSError:
             return None
         else:
             return target
 
     def _make_link_info(self, sys_path):
+        # type: (Text) -> Dict[Text, object]
         _target = self._gettarget(sys_path)
-        link = {
-            'target': _target,
-        }
-        return link
+        return {'target': _target}
 
     def getinfo(self, path, namespaces=None):
+        # type: (Text, Optional[Collection[Text]]) -> Info
         self.check()
         namespaces = namespaces or ()
         _path = self.validatepath(path)
         sys_path = self.getsyspath(_path)
         _lstat = None
         with convert_os_errors('getinfo', path):
-            _stat = os.stat(sys_path)
+            _stat = os.stat(fsencode(sys_path))
             if 'lstat' in namespaces:
-                _stat = os.lstat(sys_path)
+                _stat = os.lstat(fsencode(sys_path))
 
         info = {
             'basic': {
@@ -259,14 +285,21 @@ class OSFS(FS):
         return Info(info)
 
     def listdir(self, path):
+        # type: (Text) -> List[Text]
         self.check()
         _path = self.validatepath(path)
         sys_path = self._to_sys_path(_path)
         with convert_os_errors('listdir', path, directory=True):
-            names = os.listdir(sys_path)
-        return names
+            names = os.listdir(fsencode(sys_path))
+        return [fsdecode(name) for name in names]
+        #return names
 
-    def makedir(self, path, permissions=None, recreate=False):
+    def makedir(self,               # type: _O
+                path,               # type: Text
+                permissions=None,   # type: Optional[Permissions]
+                recreate=False      # type: bool
+                ):
+        # type: (...) -> SubFS[_O]
         self.check()
         mode = Permissions.get_mode(permissions)
         _path = self.validatepath(path)
@@ -284,6 +317,7 @@ class OSFS(FS):
             return self.opendir(_path)
 
     def openbin(self, path, mode="r", buffering=-1, **options):
+        # type: (Text, Text, int, **Any) -> BinaryIO
         _mode = Mode(mode)
         _mode.validate_bin()
         self.check()
@@ -298,9 +332,10 @@ class OSFS(FS):
                 buffering=buffering,
                 **options
             )
-        return binary_file
+        return binary_file  # type: ignore
 
     def remove(self, path):
+        # type: (Text) -> None
         self.check()
         _path = self.validatepath(path)
         sys_path = self._to_sys_path(_path)
@@ -310,15 +345,16 @@ class OSFS(FS):
             except OSError as error:
                 if error.errno == errno.EACCES and sys.platform == "win32":
                     # sometimes windows says this for attempts to remove a dir
-                    if os.path.isdir(sys_path):  # pragma: nocover
+                    if os.path.isdir(sys_path):  # pragma: no cover
                         raise errors.FileExpected(path)
                 if error.errno == errno.EPERM and sys.platform == "darwin":
                     # sometimes OSX says this for attempts to remove a dir
-                    if os.path.isdir(sys_path):  # pragma: nocover
+                    if os.path.isdir(sys_path):  # pragma: no cover
                         raise errors.FileExpected(path)
                 raise
 
     def removedir(self, path):
+        # type: (Text) -> None
         self.check()
         _path = self.validatepath(path)
         if _path == '/':
@@ -331,16 +367,27 @@ class OSFS(FS):
     # Optional Methods
     # --------------------------------------------------------
 
+    if False:  # typing.TYPE_CHECKING
+        def opendir(self, path, factory=None):
+            # type: (_O, Text, Optional[_OpendirFactory]) -> SubFS[_O]
+            pass
+
     def getsyspath(self, path):
-        sys_path = self._to_sys_path(path)
+        # type: (Text) -> Text
+        sys_path = os.path.join(
+            self._root_path,
+            path.lstrip('/').replace('/', os.sep)
+        )
         return sys_path
 
     def geturl(self, path, purpose='download'):
+        # type: (Text, Text) -> Text
         if purpose != 'download':
             raise NoURL(path, purpose)
         return "file://" + self.getsyspath(path)
 
     def gettype(self, path):
+        # type: (Text) -> ResourceType
         self.check()
         sys_path = self._to_sys_path(path)
         with convert_os_errors('gettype', path):
@@ -349,6 +396,7 @@ class OSFS(FS):
         return resource_type
 
     def islink(self, path):
+        # type: (Text) -> bool
         self.check()
         _path = self.validatepath(path)
         sys_path = self._to_sys_path(_path)
@@ -358,14 +406,16 @@ class OSFS(FS):
             return os.path.islink(sys_path)
 
     def open(self,
-             path,
-             mode="r",
-             buffering=-1,
-             encoding=None,
-             errors=None,
-             newline='',
-             line_buffering=False,
-             **options):
+             path,                  # type: Text
+             mode="r",              # type: Text
+             buffering=-1,          # type: int
+             encoding=None,         # type: Optional[Text]
+             errors=None,           # type: Optional[Text]
+             newline='',            # type: Text
+             line_buffering=False,  # type: bool
+             **options              # type: Any
+             ):
+        # type: (...) -> IO
         _mode = Mode(mode)
         validate_open_mode(mode)
         self.check()
@@ -386,6 +436,7 @@ class OSFS(FS):
             )
 
     def setinfo(self, path, info):
+        # type: (Text, RawInfo) -> None
         self.check()
         _path = self.validatepath(path)
         sys_path = self._to_sys_path(_path)
@@ -394,16 +445,17 @@ class OSFS(FS):
         if 'details' in info:
             details = info['details']
             if 'accessed' in details or 'modified' in details:
-                accessed = details.get("accessed")
-                modified = details.get("modified", accessed)
-                accessed = int(modified if accessed is None else accessed)
-                modified = int(modified)
+                _accessed = typing.cast(int, details.get("accessed"))
+                _modified = typing.cast(int, details.get("modified", _accessed))
+                accessed = int(_modified if _accessed is None else _accessed)
+                modified = int(_modified)
                 if accessed is not None or modified is not None:
                     with convert_os_errors('setinfo', path):
                         os.utime(sys_path, (accessed, modified))
 
     if scandir:
         def _scandir(self, path, namespaces=None):
+            # type: (Text, Optional[Collection[Text]]) -> Iterator[Info]
             self.check()
             namespaces = namespaces or ()
             _path = self.validatepath(path)
@@ -412,7 +464,7 @@ class OSFS(FS):
                 for dir_entry in scandir(sys_path):
                     info = {
                         "basic": {
-                            "name": dir_entry.name,
+                            "name": fsdecode(dir_entry.name),
                             "is_dir": dir_entry.is_dir()
                         }
                     }
@@ -446,20 +498,23 @@ class OSFS(FS):
     else:
 
         def _scandir(self, path, namespaces=None):
+            # type: (Text, Optional[Collection[Text]]) -> Iterator[Info]
             self.check()
             namespaces = namespaces or ()
             _path = self.validatepath(path)
-            sys_path = self._to_sys_path(_path)
+            sys_path = self.getsyspath(_path)
+            _sys_path = fsencode(sys_path)
             with convert_os_errors('scandir', path, directory=True):
                 for entry_name in os.listdir(sys_path):
-                    entry_path = os.path.join(sys_path, entry_name)
-                    stat_result = os.stat(entry_path)
+                    _entry_name = fsdecode(entry_name)
+                    entry_path = os.path.join(sys_path, _entry_name)
+                    stat_result = os.stat(fsencode(entry_path))
                     info = {
                         "basic": {
-                            "name": entry_name,
+                            "name": _entry_name,
                             "is_dir": stat.S_ISDIR(stat_result.st_mode),
                         }
-                    }
+                    }  # type: Dict[Text, Dict[Text, Any]]
                     if 'details' in namespaces:
                         info['details'] = \
                             self._make_details_from_stat(stat_result)
@@ -485,7 +540,12 @@ class OSFS(FS):
                     yield Info(info)
 
 
-    def scandir(self, path, namespaces=None, page=None):
+    def scandir(self,
+                path,               # type: Text
+                namespaces=None,    # type: Optional[Collection[Text]]
+                page=None           # type: Optional[Tuple[int, int]]
+                ):
+        # type: (...) -> Iterator[Info]
         iter_info = self._scandir(path, namespaces=namespaces)
         if page is not None:
             start, end = page
